@@ -34,46 +34,11 @@ class InferenceEngine(Module):
     inference_ep_group = None
     expert_mp_group = None
 
-    def __init__(self,
-                 model,
-                 triangular_masking=True,
-                 mp_size=1,
-                 training_mp_size=1,
-                 ep_size=1,
-                 mpu=None,
-                 ep_group=None,
-                 expert_mp_group=None,
-                 checkpoint=None,
-                 dtype=None,
-                 injection_dict=None,
-                 return_tuple=True,
-                 replace_method='auto',
-                 quantization_setting=None,
-                 replace_with_kernel_inject=False,
-                 moe=False,
-                 moe_experts=1,
-                 moe_type='standard',
-                 config=None,
-                 enable_cuda_graph=False,
-                 save_mp_checkpoint_path=None,
-                 base_dir="",
-                 max_out_tokens=1024):
+    def __init__(self, model, config):
         """
         Args:
             model: torch.nn.Module
-            mp_size: model-parallel size
-            mpu: model-parallel unit (used for Megatron-type models)
-            checkpoint: the json-path, showing the address of model-checkpoints
-                Example: {type: 'Megatron', 'checkpoints': [ckpt_mp0.pt, ckpt_mp1.pt], 'version': 1.0}
-            dtype: data-type by which inference is executed
-            injection_dict: the dictionary that shows the injection policy:
-                Example: {BertLayer: HFBertLayerPolicy}
-            return_tuple: if true, inference-API returns a tuple, otherwise a tensor
-            replace_method: the injection method, this can be passed as auto if no injection-policy is defined, in which case the injection is automatic based on the available policies
-            quantization_setting:
-                one of None, Tuple(mlp_extra_grouping, quantize_groups), quantize_groups
-            replace_with_kernel_inject: this flag need to be set to true to inject inference kernels for models such as, Bert, GPT2, GPT-Neo and GPT-J. Otherwise,
-            the injection_dict provides the names of two linear layers as a tuple: (attention_output projection, transformer output projection)
+            config: DeepSpeedInferenceConfig
         """
         global DS_INFERENCE_ENABLED
         DS_INFERENCE_ENABLED = True
@@ -81,101 +46,96 @@ class InferenceEngine(Module):
         super().__init__()
 
         self.module = model
+        self._config = config
 
-        self._get_model_config_generate(config)
+        self._get_model_config_generate(config)  # keep for weird backward compatibility
 
         if hasattr(self.module, "config"):
             DSPolicy.hf_model_config = self.module.config
 
-        self.mp_world_size = mp_size
-        self.checkpoint = checkpoint
-        self.dtype = dtype
-        self.injection_dict = injection_dict
-        self.mp_group = None
-        self.mpu = mpu
-        self._validate_args(mpu, replace_with_kernel_inject)
-        self.replace_method = replace_method
+        # todo: keep this self.injection_dict because we don't use to change config.injection_policy API
+        # todo: this will get changed when Molly's PR on auto injection dict is merged
+        self.injection_dict = config.injection_policy
+
+        # todo: refactor the mp_group and mp_size related in the next refactor
+        self.mp_group = config.tensor_parallel.tp_group
+        self.mpu = config.tensor_parallel.mpu
+
+        #self._validate_args(self.mpu, config.replace_with_kernel_inject)
         self.quantize_merge_count = 1
         self.quantization_scales = None
-        self.triangular_masking = triangular_masking
-        self.ep_size = ep_size
-        self.ep_group = ep_group
-        self.expert_mp_group = expert_mp_group
-        self.enable_cuda_graph = enable_cuda_graph
+
+        # these are not needed in the config as we are creating them ourselves in the inference engine
+        self.ep_group = None  # config.moe.ep_group
+        self.expert_mp_group = None  # config.moe.ep_mp_group
+
         self.cuda_graph_created = False
         self.checkpoint_engine = TorchCheckpointEngine()
-        self._init_quantization_setting(quantization_setting)
+        quantization_setting = None
+        self._init_quantization_setting(
+            quantization_setting
+        )  # todo: update with the new quant config for weight quant
         self.model_profile_enabled = False
         self._model_times = []
 
         # This is a hack to remove the prepare_mask function on HF side for BLOOM architecture
         self.remove_mask_prepare_for_bloom()
 
-        if enable_cuda_graph:
+        if config.enable_cuda_graph:
             assert pkg_version.parse(torch.__version__) >= pkg_version.parse("1.10"), \
                 "If you want to use cuda graph, please upgrade torch to at least v1.10"
 
-        if self.checkpoint and not replace_with_kernel_inject:
-            self._load_checkpoint(self.checkpoint)
+        if config.checkpoint and not config.replace_with_kernel_inject:
+            self._load_checkpoint(config.checkpoint)
 
         # convert model to intended dtype
-        if self.dtype:
-            self._convert_to_dtype()
+        if config.dtype:
+            self._convert_to_dtype(config)
 
         if self.mpu:
-            self.mp_world_size = dist.get_world_size(
+            config.tensor_parallel.tp_size = dist.get_world_size(
                 group=self.mpu.get_model_parallel_group())
-            self.mp_group = mpu.get_model_parallel_group()
-        elif self.mp_world_size > 1:
-            self._create_model_parallel_group()
+            self.mp_group = self.mpu.get_model_parallel_group()
+        elif config.tensor_parallel.tp_size > 1:
+            self._create_model_parallel_group(config)
+            config.tensor_parallel.tp_group = self.mp_group
 
         if isinstance(self.module, torch.nn.Module):
             moe, _ = has_moe_layers(self.module)
+        else:
+            moe = False
 
         if moe and dist.get_world_size() > 1:
-            self._create_ep_parallel_group(moe_experts)
+            self._create_ep_parallel_group(config.moe.moe_experts)
+
+        # retain this from the old conditional argument being passed to apply_injection_policy()
+        if not config.replace_with_kernel_inject:
+            config.checkpoint = None
 
         if self.injection_dict:
             for client_module, injection_policy in self.injection_dict.items():
-                self._apply_injection_policy(
-                    client_module,
-                    injection_policy,
-                    return_tuple,
-                    replace_with_kernel_inject,
-                    moe,
-                    moe_experts,
-                    moe_type,
-                    training_mp_size,
-                    self.checkpoint if replace_with_kernel_inject else None,
-                    save_mp_checkpoint_path=save_mp_checkpoint_path,
-                    base_dir=base_dir,
-                    max_out_tokens=max_out_tokens)
-        elif replace_method == 'auto':
-            self._apply_injection_policy(
-                return_tuple=return_tuple,
-                replace_with_kernel_inject=replace_with_kernel_inject,
-                moe=moe,
-                moe_experts=moe_experts,
-                moe_type=moe_type,
-                training_mp_size=training_mp_size,
-                checkpoint_dir=self.checkpoint if replace_with_kernel_inject else None,
-                save_mp_checkpoint_path=save_mp_checkpoint_path,
-                base_dir=base_dir,
-                max_out_tokens=max_out_tokens)
+                # construct the tuple and pass that instead of a string or dict.
+                if isinstance(injection_policy, str):
+                    config.injection_policy_tuple = (injection_policy, )
+                else:
+                    config.injection_policy_tuple = injection_policy
+                self._apply_injection_policy(config, client_module)
+        elif config.replace_method == 'auto':
+            self._apply_injection_policy(config)
 
         device = torch.cuda.current_device()
         self.module.to(device)
 
-        if self.mp_world_size > 1:
+        if config.tensor_parallel.tp_size > 1:
             _rng_state = torch.cuda.get_rng_state().to(torch.cuda.current_device())
             dist.broadcast(_rng_state, 0)
             torch.cuda.set_rng_state(_rng_state.cpu())
 
-        if self.mp_world_size > 1:
-            assert not self.enable_cuda_graph, "Cuda graph is not supported for model parallelism"
+        if config.tensor_parallel.tp_size > 1:
+            assert not config.enable_cuda_graph, "Cuda graph is not supported for model parallelism"
 
     def profile_model_time(self, use_cuda_events=True):
-        if not self.model_profile_enabled and not self.enable_cuda_graph:
+        if not self.model_profile_enabled and not self._config.enable_cuda_graph:
             self.module.register_forward_pre_hook(self._pre_forward_hook)
             self.module.register_forward_hook(self._post_forward_hook)
         self.model_profile_enabled = True
@@ -183,8 +143,13 @@ class InferenceEngine(Module):
         if self.use_cuda_events:
             self.timers = SynchronizedWallClockTimer()
 
+    # todo: remove this once all the config dicts are centralized from top level pydantic config
     def _get_model_config_generate(self, config):
-        self.config = getattr(self.module, 'config', None) if config is None else config
+        # this is being passed to replace_transformer_layer(config=self.user_model_config_dict)
+        self.config = getattr(self.module,
+                              'config',
+                              None) if config.config is None else config.config
+        # todo: clarify with Reza if this gets used anywhere
         self.generate = getattr(self.module, 'generate', None)
 
     def remove_mask_prepare_for_bloom(self):
@@ -209,18 +174,16 @@ class InferenceEngine(Module):
             elapsed_time = self._end - self._start
         self._model_times.append(elapsed_time)
 
-    def _create_model_parallel_group(self):
+    def _create_model_parallel_group(self, config):
         # Call the init process
         if InferenceEngine.inference_mp_group is None:
             init_distributed()
-
             local_rank = int(os.getenv('LOCAL_RANK', '0'))
             torch.cuda.set_device(local_rank)
 
-            ranks = [i for i in range(self.mp_world_size)]
+            ranks = [i for i in range(config.tensor_parallel.tp_size)]
             self.mp_group = dist.new_group(ranks)
             InferenceEngine.inference_mp_group = self.mp_group
-
         else:
             self.mp_group = InferenceEngine.inference_mp_group
 
@@ -269,26 +232,34 @@ class InferenceEngine(Module):
             f"quantize_groups = {self.quantize_groups}",
             [0])
 
+    # TODO: remove this function and add this functionality to pydantic config checking
     def _validate_args(self, mpu, replace_with_kernel_inject):
         # TODO: to support SD pipeline we need to avoid this check for now
         if replace_with_kernel_inject and not isinstance(self.module, Module):
             raise ValueError(f"model must be a torch.nn.Module, got {type(self.module)}")
-        if not isinstance(self.mp_world_size, int) or self.mp_world_size < 1:
-            raise ValueError(f"mp_size must be an int >= 1, got {self.mp_world_size}")
+        if not isinstance(self._config.tensor_parallel.tp_size,
+                          int) or self._config.tensor_parallel.tp_size < 1:
+            raise ValueError(
+                f"mp_size must be an int >= 1, got {self._config.tensor_parallel.tp_size}"
+            )
 
         if mpu:
             methods = ["get_model_parallel_group", "get_data_parallel_group"]
             for method in methods:
                 if not hasattr(mpu, method):
                     raise ValueError(f"mpu is missing {method}")
-        if self.checkpoint is not None and not isinstance(self.checkpoint, (str, dict)):
+        if self._config.checkpoint is not None and not isinstance(
+                self._config.checkpoint,
+            (str,
+             dict)):
             raise ValueError(
-                f"checkpoint must be None, str or dict, got {type(self.checkpoint)}")
+                f"checkpoint must be None, str or dict, got {type(self._config.checkpoint)}"
+            )
 
         supported_dtypes = [None, torch.half, torch.int8, torch.float]
-        if self.dtype not in supported_dtypes:
+        if self._config.dtype not in supported_dtypes:
             raise ValueError(
-                f"{self.dtype} not supported, valid dtype: {supported_dtypes}")
+                f"{self._config.dtype} not supported, valid dtype: {supported_dtypes}")
 
         if self.injection_dict is not None and not isinstance(self.injection_dict, dict):
             raise ValueError(
@@ -297,7 +268,7 @@ class InferenceEngine(Module):
     def load_model_with_checkpoint(self, r_module):
         self.mp_replace = ReplaceWithTensorSlicing(
             mp_group=self.mp_group,
-            mp_size=self.mp_world_size)  #, out_dim=0, in_dim=1)
+            mp_size=self._config.tensor_parallel.tp_size)  #, out_dim=0, in_dim=1)
         error_msgs = []
 
         def load(module, state_dict, prefix):
@@ -361,56 +332,25 @@ class InferenceEngine(Module):
 
         load_module_recursive(r_module)
 
-    def _apply_injection_policy(self,
-                                client_module=None,
-                                injection_policy=None,
-                                return_tuple=True,
-                                replace_with_kernel_inject=False,
-                                moe=False,
-                                moe_experts=1,
-                                moe_type='standard',
-                                training_mp_size=1,
-                                checkpoint_dir=None,
-                                save_mp_checkpoint_path=False,
-                                base_dir="",
-                                max_out_tokens=1024):
+    def _apply_injection_policy(self, config, client_module=None):
+        # client_module is only passed when using the injection_dict method.
+        checkpoint_dir = config.checkpoint
         checkpoint = SDLoaderFactory.get_sd_loader_json(
             checkpoint_dir,
             self.checkpoint_engine) if checkpoint_dir is not None else None
 
         generic_injection(self.module,
-                          fp16=(self.dtype == torch.half) or (self.dtype == torch.int8),
-                          enable_cuda_graph=self.enable_cuda_graph)
+                          fp16=(config.dtype == torch.half)
+                          or (config.dtype == torch.int8),
+                          enable_cuda_graph=config.enable_cuda_graph)
 
         if isinstance(self.module, torch.nn.Module):
-            replace_transformer_layer(
-                client_module,
-                self.module,
-                triangular_masking=self.triangular_masking,
-                policy=injection_policy,
-                mp_size=self.mp_world_size,
-                mp_group=self.mp_group,
-                ep_group=self.ep_group,
-                expert_mp_group=self.expert_mp_group,
-                config=self.config,
-                fp16=(self.dtype == torch.half) or (self.dtype == torch.int8),
-                training=False,
-                return_tuple=return_tuple,
-                quantize=(self.dtype == torch.int8),
-                quantize_settings=(self.quantization_scales,
-                                   self.quantize_merge_count,
-                                   self.mlp_extra_grouping,
-                                   self.quantize_groups),
-                replace_with_kernel_inject=replace_with_kernel_inject,
-                moe=moe,
-                moe_experts=moe_experts,
-                moe_type=moe_type,
-                training_mp_size=training_mp_size,
-                checkpoint_dict=checkpoint,
-                save_mp_checkpoint_path=save_mp_checkpoint_path,
-                base_dir=base_dir,
-                enable_cuda_graph=self.enable_cuda_graph,
-                max_out_tokens=max_out_tokens)
+            # config is our DeepSpeedInferenceConfig and self.config is the HF model config
+            replace_transformer_layer(client_module,
+                                      self.module,
+                                      checkpoint,
+                                      config,
+                                      self.config)
 
     def _get_all_ckpt_names(self, checkpoints_path, tag):
         ckpt_file_pattern = self._get_ckpt_name(checkpoints_path,
@@ -468,10 +408,10 @@ class InferenceEngine(Module):
         else:
             mp_rank = 0 if self.mpu is None else self.mpu.get_model_parallel_rank()
 
-            load_path, checkpoint, quantize_config = sd_loader.load(self.mp_world_size,
+            load_path, checkpoint, quantize_config = sd_loader.load(self._config.tensor_parallel.tp_size,
                                                     mp_rank,
                                                     is_pipe_parallel=is_pipe_parallel,
-                                                    quantize=(self.dtype is torch.int8),
+                                                    quantize=(self._config.dtype is torch.int8),
                                                     quantize_groups=self.quantize_groups,
                                                     mlp_extra_grouping=self.mlp_extra_grouping)
 
@@ -504,21 +444,21 @@ class InferenceEngine(Module):
         elif 'model' in sd:
             return 'model'
 
-    def _convert_to_dtype(self):
+    def _convert_to_dtype(self, config):
         if not isinstance(self.module, torch.nn.Module):
             return
 
-        if False:  #self.dtype is torch.int8 and self.quantization_scales is None:
+        if False:  #config.dtype is torch.int8 and self.quantization_scales is None:
             quantizer = WeightQuantization(mlp_extra_grouping=self.mlp_extra_grouping)
             model, self.quantization_scales = quantizer.model_quantize(self.module,
                                                                         self.injection_dict,
                                                                         self.quantize_bits,
                                                                         self.quantize_groups)
-        elif self.dtype == torch.half:
+        elif config.dtype == torch.half:
             self.module.half()
-        elif self.dtype == torch.bfloat16:
+        elif config.dtype == torch.bfloat16:
             self.module.bfloat16()
-        elif self.dtype == torch.float:
+        elif config.dtype == torch.float:
             self.module.float()
 
     def _create_cuda_graph(self, *inputs, **kwargs):
@@ -553,7 +493,7 @@ class InferenceEngine(Module):
     def model_times(self):
         assert self.model_profile_enabled, "model profiling is not enabled"
         model_times = self._model_times
-        if self.enable_cuda_graph and len(self._model_times) == 0:
+        if self._config.enable_cuda_graph and len(self._model_times) == 0:
             raise ValueError(
                 "Model times are empty and cuda graph is enabled. If "
                 "this is a GPT-style model this combo is not supported. If this is a "
@@ -570,11 +510,11 @@ class InferenceEngine(Module):
             **kwargs: variable length keyword arguments
         """
         start = None
-        if self.model_profile_enabled and self.enable_cuda_graph:
+        if self.model_profile_enabled and self._config.enable_cuda_graph:
             torch.cuda.synchronize()
             start = time.time()
 
-        if self.enable_cuda_graph:
+        if self._config.enable_cuda_graph:
             if self.cuda_graph_created:
                 outputs = self._graph_replay(*inputs, **kwargs)
             else:
@@ -583,7 +523,7 @@ class InferenceEngine(Module):
         else:
             outputs = self.module(*inputs, **kwargs)
 
-        if self.model_profile_enabled and self.enable_cuda_graph:
+        if self.model_profile_enabled and self._config.enable_cuda_graph:
             torch.cuda.synchronize()
             duration = time.time() - start
             self._model_times.append(duration)
